@@ -1,5 +1,5 @@
 package MooseX::Meta::TypeConstraint::Intersection;
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 # ABSTRACT: An intersection of Moose type constraints
 
 use Moose;
@@ -78,7 +78,7 @@ sub validate {
     my ($self, $value) = @_;
     my $msgs = $self->validate_all($value);
     return undef unless defined $msgs;
-    return join(q{ and } => @{ $msgs }) . ' in ' . $self->name;
+    return join(q{ and } => map { $_->[0] } @{ $msgs }) . ' in ' . $self->name;
 }
 
 
@@ -87,7 +87,7 @@ sub validate_all {
 
     my @msgs = map {
         my $err = $_->validate($value);
-        defined $err ? $err : ();
+        defined $err ? [ $err, $_ ] : ();
     } @{ $self->type_constraints };
 
     return @msgs ? \@msgs : undef;
@@ -129,7 +129,7 @@ MooseX::Meta::TypeConstraint::Intersection - An intersection of Moose type const
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 DESCRIPTION
 
@@ -210,9 +210,9 @@ returned by the member constraints, or C<undef>.
 
 =head2 validate_all($value)
 
-Same as C<validate>, but returns an array references of error messages from the
-individual validation errors instead of a plain string with the errors
-concatenated.
+Same as C<validate>, but returns an array reference of tuples with error
+messages and the type constraints that produced them from the individual
+validation errors instead of a plain string with the errors concatenated.
 
 =cut
 
